/*
Required functions for the Nuke Operations called from
the macroscripts.

Add core functionality in this script.

History:
1-1-2011 -- v1.0 -- loocas -- This script created

Copyright 2011 duber s.r.o., all rights reserved.
*/

/*
LICENSED UNDER THE CREATIVE COMMONS ATTRIBUTION 3.0 UNPORTED

YOU MAY: SHARE AND REMIX THE WORK UNDER THE ATTRIBUTION CONDITIONS

FOR MORE INFO: http://creativecommons.org/licenses/by/3.0/
*/

struct NukeOps
(
	nukeScriptVersion = "6.2 v1", -- adjust accordingly
	clipboardString = "", -- this value holds the whole clipboard string that the script generates
	selectionArray = #(),
	filesCount = 0,
	currentFileName = "",
	brutallyRandomString = "",
	
	fn saveToFile sel range file:undefined =
	(
		/*
		-- formatting used in the .chan file. Requires a modified import_chan_file.tcl script!
		-- transform and rotation orders required: SRT and XYZ!
		"frame pos.x   pos.y  pos.z    rot.x     rot.y     rot.z sca.x sca.y sca.z  fov    haperture  vaperture   focal_point"
		"1  -153.093  125.0  153.093  -30.071  -45.0783  2.41814  1.0  1.0  1.0    28.0     22.12      12.4425     249.464"
		*/
		
		tmpArr = #() -- a dirty hack to wrap an incoming object into an array, so I can call this function recursively, because I'm lazy typing the same shit agin and again. :)
		if classOf sel != Array do
		(
			append tmpArr sel
			sel = tmpArr
			this.selectionArray = sel
		)
		
		if sel.count > 0 do
		(
			if sel.count > 1 then
			(
				this.filesCount = 0
				msgString = "Your selection contains " + sel.count as string + " objects. Are you sure you want to generate " + sel.count as string + " individual .chan files?"
				if queryBox msgString title:"NukeOps: WARNING" then
				(
					filePath = getSavePath caption:"Chanfile Output" initialDir:(pathConfig.getCurrentProjectFolder())
					for i = 1 to sel.count do
					(
						this.saveToFile sel[i] range file:(filePath + @"\" + sel[i].name + ".chan")
					)
					print ("NukeOps: " + this.filesCount as string + "/" + sel.count as string + " .chan files successfully created.")
				)
				else
				(
					print "NukeOps: script aborted by user."
					break
				)
			)
			else
			(
				obj = sel[1]
				if file == undefined do
				(
					if (file = getSaveFileName caption:"Chanfile Output" types:"Chanfile (*.chan)|*.chan|All Files (*.*)|*.*|") == undefined do print "NukeOps: script aborted by user."; break
				)
				
				local chanfile = try(createFile file) catch(undefined) -- tries creating the specified file, if it fails, exits the script.
				
				if file != undefined and chanfile != undefined then
				(					
					this.filesCount += 1
					
					case (classOf range) of
					(
						Time:
						(
							at time currentTime
							
							if superClassOf obj == camera then
							(
								local objPos = this.convertTM obj out:#pos
								local objRot = this.convertTM obj out:#rot
								local objSca = this.convertTM obj out:#sca
								local camFocal = cameraFOV.FOVtoMM obj.fov
								local camHAperture = GetRendApertureWidth()
								local camVAperture = (GetRendApertureWidth()/GetRendImageAspect())
								local camFPoint
								if classOf obj == TargetCamera then camFPoint = obj.targetDistance else camFPoint = obj.baseObject.targetDistance
								
								format "%  %  %  %  %  %  %  %  %  %  %  %  %\n" \
								((range as integer)/ticksPerFrame) \
								objPos.x \
								objPos.y \
								objPos.z \
								objRot.x \
								objRot.y \
								objRot.z \
								objSca.x \
								objSca.y \
								objSca.z \
								camFocal \
								camHAperture \
								camVAperture \
								camFPoint \
								to:chanfile
							)
							else
							(
								local objPos = this.convertTM obj out:#pos
								local objRot = this.convertTM obj out:#rot
								local objSca = this.convertTM obj out:#sca
								
								format "%  %  %  %  %  %  %  %  %\n" \
								((range as integer)/ticksPerFrame) \
								objPos.x \
								objPos.y \
								objPos.z \
								objRot.x \
								objRot.y \
								objRot.z \
								objSca.x \
								objSca.y \
								objSca.z \
								to:chanfile							
							)
							close chanfile
						)
						Interval:
						(
							for t = range.start to range.end do
							(
								at time t
								
								if superClassOf obj == camera then
								(
									local objPos = this.convertTM obj out:#pos
									local objRot = this.convertTM obj out:#rot
									local objSca = this.convertTM obj out:#sca
									local camFocal = cameraFOV.FOVtoMM obj.fov
									local camHAperture = GetRendApertureWidth()
									local camVAperture = (GetRendApertureWidth()/GetRendImageAspect())
									local camFPoint
									if classOf obj == TargetCamera then camFPoint = obj.targetDistance else camFPoint = obj.baseObject.targetDistance
									
									format "%  %  %  %  %  %  %  %  %  %  %  %  %  %\n" \
									((t as integer)/ticksPerFrame) \
									objPos.x \
									objPos.y \
									objPos.z \
									objRot.x \
									objRot.y \
									objRot.z \
									objSca.x \
									objSca.y \
									objSca.z \
									camFocal \
									camHAperture \
									camVAperture \
									camFPoint \
									to:chanfile
								)
								else
								(
									local objPos = this.convertTM obj out:#pos
									local objRot = this.convertTM obj out:#rot
									local objSca = this.convertTM obj out:#sca
									
									format "%  %  %  %  %  %  %  %  %  %\n" \
									((t as integer)/ticksPerFrame) \
									objPos.x \
									objPos.y \
									objPos.z \
									objRot.x \
									objRot.y \
									objRot.z \
									objSca.x \
									objSca.y \
									objSca.z \
									to:chanfile							
								)
							)
							close chanfile
						)
						default: break
					)
					print ("NukeOps: .chan file successfully generated and saved to " + file)
				)
				else(print ("NukeOps: WARNING: the specified file " + file + " couldn't be written to. Skipping to next one..."))
			)
		)
	),
	
	fn copyToClipboard sel range createScene:false =
	(
		this.brutallyRandomString = ""
		this.currentFileName = (getFilenameFile maxFileName)
		local msgString = ""
		
		if sel.count > 0 then
		(
			if classOf sel == Array do qsort sel this.compareArrayObjects -- it is important to have at least one camera at the begining of the selection array!
			this.selectionArray = sel -- this variable stores the original selection for various purposes throughout the script. Mainly for counting.
			
			this.clipboardString = "set cut_paste_input [stack 0]\nversion " + nukeScriptVersion + "\npush $cut_paste_input\n"
			for o in sel do
			(
				-- this statement will filter down what will be executed further on in the script.
				case createScene of
				(
					true:
					(
						if superClassOf o != camera then
						(
							this.genNukeClipboardString o range true cameraIncluded:false
						)
						else
						(
							this.genNukeClipboardString o range true cameraIncluded:true
						)
					)
					false: this.genNukeClipboardString o range false
					default: this.genNukeClipboardString o range false
				)
			)

			
			SetClipBoardText (this.clipboardString as string) -- a simple notification system
			case (classOf range) of
			(
				Time: (msgString = ("NukeOps: " + sel.count as string + " node(s) on frame " + ((range as integer)/ticksPerFrame) as string + " successfully copied to clipboard."))
				Interval: (msgString = ("NukeOps: " + sel.count as string + " node(s) spanning " + (((range.end - range.start) as integer)/ticksPerFrame) as string + " frames successfully copied to clipboard."))
				default: ""
			)
			-- uncomment line below if you want a message box to appear every time you generate a Nuke script clipboard.
			-- messageBox msgString title:"NukeOps: SUCCESS!" beep:false
			
			-- Comment out line below if you want no feedback at all.
			print msgString
		)
		else
		(
			print "NukeOps: No objects selected."
		)
	),
	
	fn genNukeClipboardString obj range createScene cameraIncluded:false =
	(
		local numCamerasInSelection = (for o in this.selectionArray where superClassOf o == camera collect o).count
		if this.brutallyRandomString == "" do this.brutallyRandomString = ("N" + (formattedPrint (random 1111111111 9999999999) format:"+010X"))
		
		case (superClassOf obj) of
		(
			camera: if (findItem this.selectionArray obj) == 1 then this.clipboardString += "Camera2 {\n rot_order XYZ\n" else this.clipboardString += "Camera2 {\n inputs 0\n rot_order XYZ\n"
			default: this.clipboardString += "Axis2 {\n inputs 0\n rot_order XYZ"
		)
		
		case (classOf range) of
		(
			Time:
			(
				local objPos = this.convertTM obj out:#pos
				local objRot = this.convertTM obj out:#rot
				local objSca = this.convertTM obj out:#sca
				-- Build up the Nuke TCL script
				this.clipboardString += " translate {" + objPos.x as string + " " + objPos.y as string + " " + objPos.z as string + "}\n"
				this.clipboardString += " rotate {" + objRot.x as string + " " + objRot.y as string + " " + objRot.z as string + "}\n"
				this.clipboardString += " scaling {" + objSca.x as string + " " + objSca.y as string + " " + objSca.z as string + "}\n"
				if superClassOf obj == camera do
				(
					this.clipboardString += " focal " + cameraFOV.FOVtoMM obj.fov as string + "\n"
					this.clipboardString += " haperture " + GetRendApertureWidth() as string + "\n"
					this.clipboardString += " vaperture " + (GetRendApertureWidth()/GetRendImageAspect()) as string + "\n"
					if classOf obj == TargetCamera then this.clipboardString += " focal_point " + obj.targetDistance as string + "\n" else this.clipboardString += " focal_point " + obj.baseObject.targetDistance as string + "\n"
				)
				this.clipboardString += " label " + obj.name as string + "\n"
				this.clipboardString += "}\n"
				if cameraIncluded or this.selectionArray.count > 1 then
				(
					if createScene and (findItem this.selectionArray obj) == 1 do this.clipboardString += "set " + this.brutallyRandomString + " [stack 0]\n"
					if createScene and (findItem this.selectionArray obj) == (this.selectionArray.count) do
					(
						this.clipboardString += "push $" + this.brutallyRandomString + "\n"
						this.clipboardString += "Scene {\n inputs " + this.selectionArray.count as string + "\n label " + (if this.currentFileName != "" then this.currentFileName else "3dsMax_Scene") + "\n}"
						if numCamerasInSelection > 0 do this.clipboardString += "\nConstant {\n inputs 0\n}\nScanlineRender {\n inputs 3\n}"
					)
				)
				else
				(
					if createScene do this.clipboardString += "Scene {\n label " + (if this.currentFileName != "" then this.currentFileName + "_Scene" else "3dsMax_Scene") + "\n}"
				)
			)
			Interval:
			(
				if superClassOf obj == camera then -- adds an expression to the camera nodes for time freezing functionality. This is helpful when setting up projections in Nuke.
				(
					local tmpObjPosX = "{freezeTime?curve(freezeFrame):curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjPosY = "{freezeTime?curve(freezeFrame):curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjPosZ = "{freezeTime?curve(freezeFrame):curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjRotX = "{freezeTime?curve(freezeFrame):curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjRotY = "{freezeTime?curve(freezeFrame):curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjRotZ = "{freezeTime?curve(freezeFrame):curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjScaX = "{freezeTime?curve(freezeFrame):curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjScaY = "{freezeTime?curve(freezeFrame):curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjScaZ = "{freezeTime?curve(freezeFrame):curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjFocal = "{freezeTime?curve(freezeFrame):curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjFPoint = "{freezeTime?curve(freezeFrame):curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
				)
				else -- if you want to add the expression to all the nodes, including Axis, simply modify this whole if statement.
				(
					local tmpObjPosX = "{curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjPosY = "{curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjPosZ = "{curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjRotX = "{curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjRotY = "{curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjRotZ = "{curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjScaX = "{curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjScaY = "{curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjScaZ = "{curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjFocal = "{curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
					local tmpObjFPoint = "{curve i x" + ((range.start as integer)/ticksPerFrame) as string + " "
				)
				
				for i = range.start to range.end do -- go through the current active animation range and collect transformations
				(
					local objPos = at time i this.convertTM obj out:#pos
					local objRot = at time i this.convertTM obj out:#rot
					local objSca = at time i this.convertTM obj out:#sca
					
					-- POSITIONS --
					tmpObjPosX += objPos.x as string + " "
					tmpObjPosY += objPos.y as string + " "
					tmpObjPosZ += objPos.z as string + " "
					-- ROTATIONS --
					tmpObjRotX += objRot.x as string + " "
					tmpObjRotY += objRot.y as string + " "
					tmpObjRotZ += objRot.z as string + " "
					-- SCALES --
					tmpObjScaX += objSca.x as string + " "
					tmpObjScaY += objSca.y as string + " "
					tmpObjScaZ += objSca.z as string + " "
					
					if superClassOf obj == camera do
					(
						local objFocal = at time i ((cameraFOV.FOVtoMM obj.fov) as string)
						local objFPoint = if classOf obj == TargetCamera then at time i (obj.targetDistance as string) else at time i (obj.baseObject.targetDistance as string)
						
						tmpObjFocal += objFocal as string + " "
						tmpObjFPoint += objFPoint as string + " "
					)
				)
					
				-- Build up the Nuke TCL script
				this.clipboardString += " translate {" + tmpObjPosX + "} " + tmpObjPosY + "} " + tmpObjPosZ + "}}\n"
				this.clipboardString += " rotate {" + tmpObjRotX + "} " + tmpObjRotY + "} " + tmpObjRotZ + "}}\n"
				this.clipboardString += " scaling {" + tmpObjScaX + "} " + tmpObjScaY + "} " + tmpObjScaZ + "}}\n"
				if superClassOf obj == camera do
				(
					this.clipboardString += " focal {" + tmpObjFocal + "}}\n"
					this.clipboardString += " haperture " + GetRendApertureWidth() as string + "\n"
					this.clipboardString += " vaperture " + (GetRendApertureWidth()/GetRendImageAspect()) as string + "\n"
					this.clipboardString += " focal_point {" + tmpObjFPoint + "}}\n"
					this.clipboardString += " label " + obj.name as string + "\n"
					-- The code below adds a user knob only to cameras, if you want to add it to all the nodes, simply cut code below out of this statement.
					this.clipboardString += " addUserKnob {20 User l \"Time Freezer\"}" + "\n"
					this.clipboardString += " addUserKnob {6 freezeTime l \"Freeze Time?\" -STARTLINE}" + "\n"
					this.clipboardString += " addUserKnob {3 freezeFrame l \"Freeze On Frame:\" -STARTLINE}" + "\n"
					this.clipboardString += " freezeFrame " + ((currentTime as integer)/ticksPerFrame) as string + "\n"
				)
				this.clipboardString += "}\n"
				if cameraIncluded or this.selectionArray.count > 1 then
				(
					if createScene and (findItem this.selectionArray obj) == 1 do this.clipboardString += "set " + this.brutallyRandomString + " [stack 0]\n"
					if createScene and (findItem this.selectionArray obj) == (this.selectionArray.count) do
					(
						this.clipboardString += "push $" + this.brutallyRandomString + "\n"
						this.clipboardString += "Scene {\n inputs " + this.selectionArray.count as string + "\n label " + (if this.currentFileName != "" then this.currentFileName else "3dsMax_Scene") + "\n}"
						if numCamerasInSelection > 0 do this.clipboardString += "\nConstant {\n inputs 0\n}\nScanlineRender {\n inputs 3\n}"
					)
				)
				else
				(
					if createScene do this.clipboardString += "Scene {\n label " + (if this.currentFileName != "" then this.currentFileName + "_Scene" else "3dsMax_Scene") + "\n}"
				)
			)
			default: this.clipboardString += "" -- should never occur, though.
		)
	),
	
	fn convertTM obj out:undefined = -- a function used for converting 3ds Max's transform matirx to Nuke's transformation worl
	(
		nukeIDMatrix = RotateXMatrix(90) -- same as Maya
		inputMatrix = preRotateX (orthogonalize(obj.transform)) -90 -- if you want to keep the same local orientation of all nodes in Nuke as they were in Max, keep it.
		-- however, while this is not important for Axis, it's essential for cameras as they look along a different vector than Nuke or Maya!
		outputMatrix = xFormMat inputMatrix nukeIDMatrix
		
		case out of -- based on what you want returned. I added this for more flexible usage outside of the scope of this tool.
		(
			#pos: return (outputMatrix.translationPart)
			#rot: return (quatToEuler (outputMatrix.rotationPart))
			#sca: return (outputMatrix.scalePart)
			default: return outputMatrix
		)
	),
	
	fn compareArrayObjects obj1 obj2 = -- a function used to reorganise the selection array and place all cameras at the begining of the array
	(
		case of
		(
			((superClassOf obj1) == camera and (superClassOf obj2) == camera and (superClassOf obj1) == (superClassOf obj2)): 0
			((superClassOf obj1) == camera and (superClassOf obj2) != camera): -1
			((superClassOf obj1) != camera and (superClassOf obj2) == camera): 1
			default: 1
		)
	)
)

NukeOps = NukeOps() -- required instantination