## -*- mode: tcl -*-
# Copyright (c) 2007 The Foundry Visionmongers Ltd.  All Rights Reserved.

# Read a .chan file into the current node which should be a Camera or
# Axis.

# read data from a camera chan file:
# format is: FRAME TX TY TZ RX RY RZ VFOV
# If file contains fewer columns than only those columns are read
# 
# Modified:
# 1-1-2011 -- v1.0 -- loocas -- Added some code to accustom to the NukeOps .chan output


proc import_chan_file {filename} {

  set not_camera ![exists this.vaperture]

  if !$not_camera {
      set va [value vaperture]
      knob this.rot_order [lindex 0]
  }

  catch {animation translate.x clear}
  catch {animation translate.y clear}
  catch {animation translate.z clear}
  catch {animation rotate.x clear}
  catch {animation rotate.y clear}
  catch {animation rotate.z clear}
  catch {animation scaling.x clear}
  catch {animation scaling.y clear}
  catch {animation scaling.z clear}
  catch {animation focal clear}
  catch {animation haperture clear}
  catch {animation vaperture clear}
  catch {animation focal_point clear}

  set f [open $filename r]
  set n 0
  while {[gets $f line]>=0} {
      if {[llength $line] < 4} continue
      incr n
      set fr [lindex $line 0]
      setkey translate.x $fr [lindex $line 1]
      setkey translate.y $fr [lindex $line 2]
      setkey translate.z $fr [lindex $line 3]
      if {[llength $line] < 7} continue
      setkey rotate.x $fr [lindex $line 4]
      setkey rotate.y $fr [lindex $line 5]
      setkey rotate.z $fr [lindex $line 6]
      if {[llength $line] < 10} continue
      setkey scaling.x $fr [lindex $line 7]
      setkey scaling.y $fr [lindex $line 8]
      setkey scaling.z $fr [lindex $line 9]
      
      if {[llength $line] < 13} continue
      knob this.haperture [lindex $line 11]
      knob this.vaperture [lindex $line 12]

      if $not_camera continue
      setkey focal $fr [lindex $line 10]
      setkey focal_point $fr [lindex $line 13]


  }
  close $f

  # Turn off selectable flag so user doesn't destroy animation accidently:
  catch {knob selected false; knob selectable false}

  # Save the filename into the node's label so user knows
  # where data came from.  This concats onto the existing label currently.
  catch {knob label [file tail $filename]}

  return "Loaded $n frames."
}

